@@ -71,7 +71,7 @@ def _add_date_sql(self: Hive.Generator, expression: DATE_ADD_OR_SUB) -> str:
         multiplier *= -1
 
     if expression.expression.is_number:
-        modified_increment = exp.Literal.number(int(expression.text("expression")) * multiplier)
+        modified_increment = exp.Literal.number(expression.expression.to_py() * multiplier)
     else:
         modified_increment = expression.expression
         if multiplier != 1:
@@ -446,12 +446,13 @@ class Hive(Dialect):
         JSON_PATH_SINGLE_QUOTE_ESCAPE = True
         SUPPORTS_TO_NUMBER = False
         WITH_PROPERTIES_PREFIX = "TBLPROPERTIES"
+        PARSE_JSON_NAME = None
 
         EXPRESSIONS_WITHOUT_NESTED_CTES = {
             exp.Insert,
             exp.Select,
             exp.Subquery,
-            exp.Union,
+            exp.SetOperation,
         }
 
         SUPPORTED_JSON_PATH_PARTS = {
@@ -575,7 +576,6 @@ class Hive(Dialect):
             exp.NotForReplicationColumnConstraint: lambda *_: "",
             exp.OnProperty: lambda *_: "",
             exp.PrimaryKeyColumnConstraint: lambda *_: "PRIMARY KEY",
-            exp.ParseJSON: lambda self, e: self.sql(e.this),
             exp.WeekOfYear: rename_func("WEEKOFYEAR"),
             exp.DayOfMonth: rename_func("DAYOFMONTH"),
             exp.DayOfWeek: rename_func("DAYOFWEEK"),
