@@ -689,6 +689,7 @@ class MySQL(Dialect):
         JSON_PATH_BRACKETED_KEY_SUPPORTED = False
         JSON_KEY_VALUE_PAIR_SEP = ","
         SUPPORTS_TO_NUMBER = False
+        PARSE_JSON_NAME = None
 
         TRANSFORMS = {
             **generator.Generator.TRANSFORMS,
@@ -714,7 +715,6 @@ class MySQL(Dialect):
             exp.Month: _remove_ts_or_ds_to_date(),
             exp.NullSafeEQ: lambda self, e: self.binary(e, "<=>"),
             exp.NullSafeNEQ: lambda self, e: f"NOT {self.binary(e, '<=>')}",
-            exp.ParseJSON: lambda self, e: self.sql(e, "this"),
             exp.Pivot: no_pivot_sql,
             exp.Select: transforms.preprocess(
                 [
@@ -1093,29 +1093,6 @@ class MySQL(Dialect):
             "xor",
             "year_month",
             "zerofill",
-            "cume_dist",
-            "dense_rank",
-            "empty",
-            "except",
-            "first_value",
-            "grouping",
-            "groups",
-            "intersect",
-            "json_table",
-            "lag",
-            "last_value",
-            "lateral",
-            "lead",
-            "nth_value",
-            "ntile",
-            "of",
-            "over",
-            "percent_rank",
-            "rank",
-            "recursive",
-            "row_number",
-            "system",
-            "window",
         }
 
         def array_sql(self, expression: exp.Array) -> str:
